PrependTo[$Path, "../.."]

Needs["HEPMath`"]

DeclareLorentzVectors[p1, p2]

approxEqual[a_, b_] := Abs[a-b] < Max[10.0^(-14)*Max[Abs[a],Abs[b]], 10.0^(-14)]

testTwoBodyDecay[] := Block[
  {tests={}, m0, m1, m2, theta, phi, n, p1, p2, E1, E2, p},

  m0 = 10.;
  m1 = 1.;
  m2 = 2.;
  theta = 0.3 Pi;
  phi = 1.7 Pi;

  n = {10., 0.4, 1.2, 0.7};
  n = n/Sqrt[MProd[n,n]];

  {p1, p2} = TwoBodyDecay[m0*n, m1, m2, theta, phi];

  p = Sqrt[KineticLambda[m0^2, m1^2, m2^2]]/(2 m0);
  E1 = Sqrt[p^2 + m1^2];
  E2 = Sqrt[p^2 + m2^2];
  
  AppendTo[tests, approxEqual[MProd[p1,p2], E1 E2 + p^2]];
  
  And @@ tests
]

testPol[] := Block[
  {tests={}, vec={2.5, 1.3, 0.7}, k, p, m=10., ek, ekc, ep, epc},

  k = Prepend[vec, Sqrt[vec.vec]];
  p = Prepend[vec, Sqrt[m^2 + vec.vec]];

  Do[
    ek = Pol[k][sg];
    ekc = CC[Pol[k]][sg];
    AppendTo[tests, approxEqual[MProd[ek, k], 0.]];
    AppendTo[tests, approxEqual[MProd[ekc, k], 0.]]
    ,
    {sg,{-1,1}}
  ];

  Do[
    ep = MPol[p][sg];
    epc = CC[MPol[p]][sg];
    AppendTo[tests, approxEqual[MProd[ep, p], 0.]];
    AppendTo[tests, approxEqual[MProd[epc, p], 0.]]
    ,
    {sg,{-1,0,1}}
  ];

  And @@ tests
]


fail = False;

WriteString[$Output, "testTwoBodyDecay: "<>If[testTwoBodyDecay[],"PASS\n", fail=True; "FAIL\n"]];
WriteString[$Output, "testPol:          "<>If[testPol[],"PASS\n", fail=True; "FAIL\n"]];

If[fail, Exit[1], Exit[]];

