PrependTo[$Path, "../.."]

Needs["HEPMath`"]

Attributes[quietCheck] = {HoldAll};
Attributes[quietCheckAbort] = {HoldAll};

quietCheck[ex_, msg_] :=
  CheckAbort[Quiet[Check[ex; False, True, msg], msg], False];
quietCheckAbort[ex_, msg_] :=
  Quiet[Check[CheckAbort[ex; False, True]; False, True, msg], msg]

testExpand[] := Block[{p,q,k,e,mu,nu,al,bt,a,b,c,expr,tests={}},
  Scope[
    DeclareLorentzVectors[p, q, k];
    DeclareComplexLorentzVectors[e];

    expr = Gs[(a+b)(p+q)].(PL-PR).Gs[k-q];
    AppendTo[tests,HEPExpand[expr,_?DiracMatrixQ]===
      Gs[(a+b)(p+q)].PL.Gs[k-q] - Gs[(a+b)(p+q)].PR.Gs[k-q]];
    AppendTo[tests,HEPExpand[expr,_?LorentzVectorQ]===
      (a+b)Gs[p].(PL-PR).Gs[k] - (a+b)Gs[q].(PL-PR).Gs[q] -
      (a+b)Gs[p].(PL-PR).Gs[q] + (a+b)Gs[q].(PL-PR).Gs[k]];
    AppendTo[tests,HEPExpand[expr,p]===
      (a+b)Gs[p].(PL-PR).Gs[k-q] + Gs[(a+b) q].(PL-PR).Gs[k-q]];
    AppendTo[tests,HEPExpand[expr,p|PL]===
      (a+b)Gs[p].PL.Gs[k-q] + Gs[(a+b) q].PL.Gs[k-q] +
      (a+b)Gs[p].(-PR).Gs[k-q] + Gs[(a+b) q].(-PR).Gs[k-q]];
    AppendTo[tests,HEPExpand[(a p + b q + c k)[mu],q] ===
      (a p)[mu] + (c k)[mu] + b q[mu]];
    AppendTo[tests,HEPExpand[(a (Gs[p] + Gs[q]))[al,bt]] ===
      a Gs[p][al,bt] + a Gs[q][al,bt]];
  ];

  And@@tests
]

testConjugation[] := Block[{p,k,e,mu,nu,r,s,a,b,c,m,expr1,expr2,tests={}},
  Scope[
    DeclareLorentzVectors[p,q,k];
    DeclareComplexLorentzVectors[e];
    DeclareReal[{a,b,m}];
  
    expr1 = Bar[USp[p][r]].Gs[2 I p + 3 q].(a PL + b PR).(c G5).VSp[p][s] e[mu];
    expr2 = Bar[USp[k][r]].Ga[mu].(Gs[q] + m GI).Ga[nu].VSp[p][s];
    AppendTo[tests, CC[expr1] === CC[e][mu] *
      Bar[VSp[p][s]].(-CC[c] G5).(a PR + b PL).Gs[3 q - 2 I p].USp[p][r]];
    AppendTo[tests, CC[expr2] ===
      Bar[VSp[p][s]] . Ga[nu] . (GI m + Gs[q]) . Ga[mu] . USp[k][r]];
    AppendTo[tests, CC[EpsDot[p,q,k,e]] === EpsDot[p,q,k,CC[e]]];
  ];
  And@@tests
]

testDiracContract[] := Block[{p,k,q,m,al,r,s,expr1,expr2,tests={}},
  Scope[
    DeclareLorentzVectors[k,p,q];

    expr1 = Bar[USp[k][r]].Ga[mu].(Gs[q] + m GI).Ga[nu].VSp[p][s] //
      HEPExpand // ExplicitIndices // DiracContract;
    expr2 = (Ga[mu].Gs[q].Ga[nu])[al,al] //
      ExplicitIndices // DiracContract;

    AppendTo[tests,  expr1 === m Bar[USp[k][r]] . Ga[mu] . Ga[nu] . VSp[p][s] +
      Bar[USp[k][r]] . Ga[mu] . Gs[q] . Ga[nu] . VSp[p][s]];
    AppendTo[tests,
      expr2 === DiracTr[Ga[mu].Gs[q].Ga[nu]] ||
      expr2 === DiracTr[Ga[nu].Ga[mu].Gs[q]] ||
      expr2 === DiracTr[Gs[q].Ga[nu].Ga[mu]]];
    AppendTo[tests, MatchQ[ExplicitSlash[Gs[q]], q[mu_] Ga[mu_]]];
  ];
  And@@tests
]

testLorentzContract[] := Block[{p,k,q,l,al,bt,ga,dl,tests={}},
  Scope[
    Declare4DLorentzVectors[k,p,q];
    DeclareLorentzVectors[l];

    AppendTo[tests, quietCheckAbort[EpsDot[k,p,q,l,k], EpsDot::toomany]];
    AppendTo[tests, quietCheckAbort[HEPTensorSignature[EpsDot[k,p,mu]],
      EpsDot::invarg]];
    AppendTo[tests, EpsDot[][mu, nu, ro, sg] === Eps[mu, nu, ro, sg]];
    AppendTo[tests, LorentzContract[Eps[mu, nu, mu, sg]] === 0];
    AppendTo[tests, LorentzContract[EpsDot[p][mu, nu, mu]] === 0];
    AppendTo[tests, LorentzContract[EpsDot[p, p][mu, nu]] === 0];
    AppendTo[tests, LorentzContract[Eps[mu, nu, ro, sg] p[ro] p[nu]] === 0];
    AppendTo[tests, LorentzContract[Eps[mu, nu, ro, sg] p[ro] q[nu]] ===
      -EpsDot[p, q][mu, sg]];
    AppendTo[tests, LorentzContract[EpsDot[k, l][mu, nu] p[mu] p[nu]] === 0];
    AppendTo[tests, LorentzContract[Eps[mu, nu, ro, sg] EtaHat[sg, lm]] === 0];
    AppendTo[tests, LorentzContract[EpsDot[p][mu, nu, ro] EtaHat[ro, lm]] === 0];

    AppendTo[tests, LorentzContract[EtaHat[mu, nu] p[nu]] === 0];
    AppendTo[tests, LorentzContract[EtaHat[mu, nu] l[mu] l[nu]] === HDot[l, l]];
    AppendTo[tests, LorentzContract[Eta[mu, nu] p[mu] q[nu]] === SDot[p, q]];
    AppendTo[tests, LorentzContract[p[mu] q[mu]] === SDot[p, q]];
    AppendTo[tests, LorentzContract[Eta[mu, nu] EtaHat[nu, mu]] === Dim - 4];
    AppendTo[tests, LorentzContract[Eta[mu, mu]] === Dim];
    AppendTo[tests, LorentzContract[p[mu] p[mu]] === Sqr[p]];
  ];
  And@@tests
]

testPullVectors[] := Block[{p,k,q,l,al,bt,ga,dl,tests={}},
  Scope[
    Declare4DLorentzVectors[k,p,q];
    DeclareLorentzVectors[l];
    DeclareComplexLorentzVectors[e];

    AppendTo[tests, MatchQ[PullVectors[EpsDot[k, l, p, q], l],
      HoldPattern[EpsDot[k, p, q][mu_]] l[mu_]]];
    AppendTo[tests, MatchQ[PullVectors[EpsDot[k, l, p, q], p],
      -HoldPattern[EpsDot[k, l, q][mu_]] p[mu_]]];
    AppendTo[tests, MatchQ[PullVectors[EpsDot[k, l, p, q], p|l],
      (EpsDot[k, q][mu_, nu_] l[mu_] p[nu_] ) |
      (-EpsDot[k, q][mu_, nu_] l[nu_] p[mu_])]];
    AppendTo[tests, MatchQ[PullVectors[EpsDot[k, l, p][mu], l],
      (-EpsDot[k, p][mu_, nu_] l[mu_]) |
      (EpsDot[k, p][mu_, nu_] l[mu_])]];

    AppendTo[tests, MatchQ[PullVectors[SDot[k, l] + Sqr[l], l],
      k[mu_] l[mu_] + l[nu_]^2]];
    AppendTo[tests, MatchQ[PullVectors[DiracTr[Ga[mu].Gs[l].Ga[nu]], l],
      DiracTr[Ga[mu].Ga[ro_].Ga[nu]] l[ro_]]];
    AppendTo[tests, MatchQ[PullVectors[HDot[k, l], l],
      EtaHat[mu_, nu_] k[mu_] l[nu_]]];

  ];
  And@@tests
]

testPolarizationSums[] := Block[{p,k,n,al,bt,mu,nu,mk,mp,s1,s2,mat,tests={}},
  Scope[
    DeclareLorentzVectors[k,p,n];

    mat = Bar[USp[k]][s1].Ga[mu].USp[p][s2];
    AppendTo[tests, FermionPolarizationSums[
      HEPMultiply[mat, CC[mat]], {k -> mk, p -> mp}] ===
      DiracTr[(Gs[k] + GI*mk) . Ga[mu] . (Gs[p] + GI*mp) . Ga[mu]]];
    AppendTo[tests, FermionPolarizationSums[
      USp[p][s1, al] Bar[USp[p]][s1, bt], p -> mp] ===
      (Gs[p] + GI*mp)[al, bt]];

    mat = Bar[VSp[k]][s1].Ga[mu].VSp[p][s2];
    AppendTo[tests, FermionPolarizationSums[
      HEPMultiply[mat, CC[mat]], {k -> mk, p -> mp}] ===
      DiracTr[(Gs[k] - GI*mk) . Ga[mu] . (Gs[p] - GI*mp) . Ga[mu]]];
    AppendTo[tests, FermionPolarizationSums[
      VSp[p][s1, al] Bar[VSp[p]][s1, bt], p -> mp] ===
      (Gs[p] - GI*mp)[al, bt]];

    AppendTo[tests, VectorPolarizationSums[
      MPol[p][s1,mu] CC[MPol[p]][s1,nu], {p -> mp}] ===
      -Eta[mu,nu] + p[mu] p[nu]/mp^2];
    AppendTo[tests, VectorPolarizationSums[
      Pol[p][s1,mu] CC[Pol[p]][s1,nu], {p -> n}] ===
      -Eta[mu,nu] - Sqr[n] p[mu] p[nu]/SDot[p,n]^2 +
      (n[mu] p[nu] + p[mu] n[nu])/SDot[p,n]];
    AppendTo[tests, VectorPolarizationSums[
      Pol[p][s1,mu] CC[Pol[p]][s1,nu], {p -> 0}] === -Eta[mu,nu]];
  ];
  And@@tests
]


fail = False;

WriteString[$Output, "testExpand:           "<>If[testExpand[],"PASS\n", fail=True; "FAIL\n"]];
WriteString[$Output, "testConjugation:      "<>If[testConjugation[],"PASS\n", fail=True; "FAIL\n"]];
WriteString[$Output, "testDiracContract:    "<>If[testDiracContract[],"PASS\n", fail=True; "FAIL\n"]];
WriteString[$Output, "testLorentzContract:  "<>If[testLorentzContract[],"PASS\n", fail=True; "FAIL\n"]];
WriteString[$Output, "testPullVectors:      "<>If[testPullVectors[],"PASS\n", fail=True; "FAIL\n"]];
WriteString[$Output, "testPolarizationSums: "<>If[testPolarizationSums[],"PASS\n", fail=True; "FAIL\n"]];

If[fail, Exit[1], Exit[]];


